
function [cpts_out,value]=parm_unconstrained_optimization_disk(deg,...
    cpts_in,family_type,mildness,verbosity)


%--------------------------------------------------------------------------
% INPUTS.
%---------
% deg: DEGREE OF THE FAMILY OF POINTS.
%
% cpts_in: SIMPLEX POINTS. THEY ARE STORED AS A VECTOR OF ANGLES AND RADII.
%
% family_type: 1: VANDERMONDE. 2: LEBESGUE CONST. OTHER: RIESZ POINTS.
%
% mildness: IT DECIDES THE MINIMIZER options. THE HIGHER THE STRONGER.
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
% cpts_out: DISK POINTS AFTER THE OPTIMIZATION PROCESS. THEY ARE STORED
%   AS A VECTOR OF ANGLES AND RADII.
%--------------------------------------------------------------------------
%%
%% Copyright (C) 2010. Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Matteo Briani
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: January 11, 2011
%%
%--------------------------------------------------------------------------

% DATA BEFORE STARTING THE PROCESS.
patts=disk_patterns(deg);
if size(cpts_in,1) == 0
    L=length(patts);
    if rem(deg,2) == 0
        angles_loc=2*pi*rand(L-1,1);
        radii_loc=sort(rand(L-2,1));
    else
        angles_loc=2*pi*rand(L,1);
        radii_loc=sort(rand(L-1,1));
    end
    x_in=[angles_loc; radii_loc];
else
    x_in=cpts_in;
end

% OPTIONS.
[options,degree_leb,maxfuns,maxtol,tolx]=option_settings_constrained(...
    deg,mildness,verbosity);
switch family_type
    case 2
        pts_leb=disk_wam(degree_leb);
        Vleb= koornwinder_typeII(deg,pts_leb,-0.5);
    otherwise
        Vleb=[];
end

%----------------------------------
% OPTIMIZATION PROCESS.
%----------------------------------
A=[]; B=[]; Aeq=[]; Beq=[]; LB=[]; UB=[];
time1=cputime;

if verbosity == 0
    x_out = fminsearch(@(x) opt_target(x,deg,family_type,patts,Vleb),...
        x_in,options);
else
    x_out = fminunc(@(x) opt_target(x,deg,family_type,patts,Vleb),...
        x_in,options);
end
time2=cputime;

%----------------------------------
% SEE IF THERE ARE IMPROVEMENTS.
%----------------------------------

switch family_type

    case 1
        value_in=abs(opt_target(x_in,deg,family_type,Vleb));
        value_out=abs(opt_target(x_out,deg,family_type,Vleb));
        % fprintf('\n \t %% IN: %2.5e OUT: %2.5e',value_in,value_out);
        if value_in > value_out
            x_out=x_in;
            value=value_in;
        else
            value=value_out;
        end

    case 2
        pts_leb=disk_wam(250);
        V_leb= koornwinder_typeII(deg,pts_leb,-0.5);

        [pts_in,err]=decoding_points(x_in,patts);
        V_pts_in=koornwinder_typeII(deg,pts_in,-0.5);
        value_in=norm(V_pts_in'\V_leb',1);

        [pts_out,err]=decoding_points(x_out,patts);
        V_pts_out = koornwinder_typeII(deg,pts_out,-0.5);
        value_out=norm(V_pts_out'\V_leb',1);

        % fprintf('\n \t %% [%1.0f][%1.3e] VALUE IN: %2.5e OUT: %2.5e',...
        %    mildness,time2-time1,value_in,value_out)

        if value_in < value_out
            x_out=x_in; value=value_in;
        else
            value=value_out;
        end

    otherwise

        value_in= opt_target(x_in,deg,family_type,Vleb);
        value_out= opt_target(x_out,deg,family_type,Vleb);

        if abs(value_in) < abs(value_out)
            x_out=x_in;
            value=value_in;
        else
            value=value_out;
        end

end

%----------------------------------
% REASSEMBLING POINTS.
%----------------------------------
pts_out=[x_out(1:dim_in,1) x_out(dim_in+1:end,1)];






















%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES.
%--------------------------------------------------------------------------

function [options,lebdeg,maxfuns,maxtol,tolx]=option_settings_unconstrained(...
    deg,mildness,verbosity)

switch mildness
    case 0
        if deg < 7
            lebdeg=70;
        else
            if deg < 12
                lebdeg=50;
            else
                lebdeg=30;
            end
        end
        maxfuns=50; maxtol=10^(-12); tolx=maxtol;
    case 1

        if deg < 7
            lebdeg=80;
        else
            if deg < 12
                lebdeg=60;
            else
                lebdeg=40;
            end
        end

        maxfuns=50; maxtol=10^(-12);  tolx=maxtol;
    case 2
        if deg < 7
            lebdeg=100;
        else
            if deg < 12
                lebdeg=70;
            else
                lebdeg=50;
            end
        end
        maxfuns=50; maxtol=10^(-12); tolx=maxtol;
    case 3
        if deg < 7
            lebdeg=135;
        else
            if deg < 12
                lebdeg=85;
            else
                lebdeg=60;
            end
        end
        maxfuns=50; maxtol=10^(-12); tolx=maxtol;
    case 4
        if deg < 7
            lebdeg=170;
        else
            if deg < 12
                lebdeg=120;
            else
                lebdeg=80;
            end
        end
        maxfuns=50; maxtol=10^(-14); tolx=maxtol;

    case 5

        if deg < 7
            lebdeg=250;
        else
            if deg < 12
                lebdeg=150;
            else
                lebdeg=100;
            end
        end
        maxfuns=25; maxtol=10^(-14);  tolx=10^(-60);

    case 6
        if deg < 7
            lebdeg=270;
        else
            if deg < 12
                lebdeg=170;
            else
                lebdeg=120;
            end
        end
        maxfuns=25; maxtol=10^(-14); tolx=maxtol;

    case 7

        if deg < 7
            lebdeg=300;
        else
            if deg < 12
                lebdeg=200;
            else
                lebdeg=140;
            end
        end
        maxfuns=25; maxtol=10^(-14);  tolx=10^(-60);

    case 8

        if deg < 7
            lebdeg=325;
        else
            if deg < 12
                lebdeg=250;
            else
                lebdeg=170;
            end
        end
        maxfuns=25; maxtol=10^(-14);  tolx=10^(-60);

    otherwise

        if deg < 7
            lebdeg=350;
        else
            if deg < 12
                lebdeg=250;
            else
                lebdeg=170;
            end
        end
        maxfuns=50; maxtol=10^(-400);  tolx=10^(-60);

end

switch verbosity

    case  0 % fminsearch/fminunc basics.
        options = optimset('MaxIter',10,'MaxFunEvals',1000*maxfuns,...
            'TolX',tolx,'TolFun',tolx,'Display','off');

    case -1 % BFGS (QUIET).
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',1000*maxfuns,...
            'TolX',tolx,'TolFun',tolx,'Display','off','LargeScale','off',...
            'Hessian','off','HessUpdate','bfgs');

    case 1 % BFGS.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',1000*maxfuns,...
            'TolX',tolx,'TolFun',tolx,'Display','iter','LargeScale','off',...
            'Hessian','off','HessUpdate','bfgs');

    case  -2 % DFP (QUIET).
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',1000*maxfuns,...
            'TolX',tolx,'TolFun',tolx,'Display','off','LargeScale','off',...
            'Hessian','off','HessUpdate','dfp');

    case  2 % DFP.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',1000*maxfuns,...
            'TolX',tolx,'TolFun',tolx,'Display','iter','LargeScale','off',...
            'Hessian','off','HessUpdate','dfp');

    case -3 % STEEPDESC.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',1000*maxfuns,...
            'TolX',tolx,'TolFun',tolx,'Display','off','LargeScale','off',...
            'Hessian','off','HessUpdate','steepdesc');

    case 3 % STEEPDESC.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',1000*maxfuns,...
            'TolX',tolx,'TolFun',tolx,'Display','iter','LargeScale','off',...
            'Hessian','off','HessUpdate','steepdesc');


        % HessUpdate
        % Method 'bfgs' 'dfp' 'steepdesc'

        % Hessian Update
        % Initial quasi-Newton matrix. This option is only available if you set InitialHessType to 'user-supplied'.
        % In that case, you can set InitialHessMatrix to one of the following:
        % scalar ? the initial matrix is the scalar times the identity
        % vector ? the initial matrix is a diagonal matrix with the entries of the vector on the diagonal.InitialHessType
        % Initial quasi-Newton matrix type. The options are
        % 'identity'
        % 'scaled-ident

end



function value=opt_target(x,deg,family_type,patts,Vleb)

%--------------------------------------------------------------------------
% INPUTS.
%---------
% x: DISK POINTS ARE STORED IN THE VECTOR x. IF THEY ARE "[X Y]"
%   (N x 2 MATRIX) THEN "x=[X; Y]" (2*N x 1 MATRIX).
%
% deg: DEGREE OF THE FAMILY OF POINTS.
%
% family_type: 1: VANDERMONDE. 2: LEBESGUE CONST.
%
% Vleb: VAND. MATRIX ON SETS THE DEGREE OF A FAMILY OF POINTS OVER WHICH
%    THE LEBESGUE CONSTANT IS TESTED.
%
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
% value: VALUE OF THE DETERMINANT OR OF THE LEBESGUE CONSTANT (SEE family_type).
%--------------------------------------------------------------------------
% ROUTINES.
%----------
% 1. dubiner_leg (AND ITS SUBROUTINES!).
% 2. blyth_pozrikidis.
%--------------------------------------------------------------------------
%%
%% Copyright (C) 2010. Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Matteo Briani
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: October 24, 2010
%%
%--------------------------------------------------------------------------

%---------------------------------------------
% REASSEMBLING OPTIMIZATION VECTOR INTO N x 2
% MATRIX.
%--------------------------------------------
[pts,err]=decoding_points(x,patts);

X=pts(:,1);
Y=pts(:,2);

%---------------------------------------------
% OPTIMIZATION TARGET FUNCTION..
%---------------------------------------------

s = (X.^2 + Y.^2 - 1 > 10^(-14)); sum_s=sum(s);

if sum_s > 0 | (err == 1)

    % fprintf('\n \t %% WARNING: ITERATION OUT OF DOMAIN.');

    switch family_type

        case -2
            value=10^30; return;

        case -1
            value=10^30; return;

        case 0
            value=10^30; return;

        case 1 % VANDERMONDE DETERMINANT: FEKETE POINTS.
            value=10^(-30); return;

        case 2 % APPROXIMATE LEBESGUE CONSTANT: APPROXIMATE LEBESGUE POINTS.
            value=10^30; return;

    end
end

V = koornwinder_typeII(deg,pts,-0.5);

switch family_type

    case -2
        s=-family_type;
        value=riesz_target(pts,s);

    case -1
        s=-family_type;
        value=riesz_target(pts,s);

    case 0
        s=family_type;
        value=riesz_target(pts,s);

    case 1 % VANDERMONDE DETERMINANT: FEKETE POINTS.
        detV=det(V);
        value=-abs(detV);
        %fprintf('\n \t %% %2.20e',value);

    case 2 % APPROXIMATE LEBESGUE CONSTANT: APPROXIMATE LEBESGUE POINTS.
        value=norm(V'\Vleb',1);

end



%---------------------------------------------
% NAN MANAGEMENT.
% IF NAN OCCURS SOME DEFAULTS ARE SET SO THAT
% THE ROUTINE KEEPS WORKING
%---------------------------------------------
inan=isnan(value); iinf=isinf(value);
if inan == 1 | iinf == 1
    % fprintf('\n \t %% [SINGULARITY]');
    switch family_type
        case 1
            value=10^(-15);
        otherwise
            value=10^50;
    end
end




function [c,ceq]=nonlcon(x)

L=length(x)/2;
xx=x(1:L); yy=x(L+1:end);
c=xx.^2+yy.^2-1;
ceq=[];

